package com.ibm.oti.shared;

import java.lang.ref.WeakReference;
import java.security.AccessControlException;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

/**
 * Abstract SharedHelper provides common functions and data to helper subclasses.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedHelper
 * @see SharedClassAbstractHelper
 */
public abstract class SharedAbstractHelper implements SharedHelper {
	private Boolean verbose;
	private WeakReference loaderRef;
	private SharedClassPermission readPerm, writePerm;

	boolean canFind, canStore;
	int id;

	void initialize(ClassLoader loader, int id, boolean canFind, boolean canStore) {
		this.id = id;
		this.canFind = canFind;
		this.canStore = canStore;
		loaderRef = new WeakReference(loader);
		printVerboseInfo("Created "+getHelperType()+" with id "+id);
	}

	/**
	 * Utility function. Returns the ClassLoader that owns this SharedHelper.
	 * Will return null if the Helper's ClassLoader has been Garbage Collected.
	 * <p>
	 * @return 		The ClassLoader owning this helper, or null if the ClassLoader has been GC'd
	 */
	public ClassLoader getClassLoader() {
		return (ClassLoader)loaderRef.get();
	}

	private native boolean getIsVerboseImpl();

	/* Do not cache the permission objects, else classloader references will prevent class GC */
	private boolean checkPermission(SecurityManager sm, ClassLoader loader, String type) {
		boolean result = false;
		try {
			sm.checkPermission(new SharedClassPermission(loader, type));
			result = true;
		} catch (AccessControlException e) {
		}
		return result;
	}

	boolean checkReadPermission() {
		SecurityManager sm = System.getSecurityManager();
		if (sm!=null) {
			return checkPermission(sm, getClassLoader(), "read");
		}
		return true; // no security manager means the check is successful
	}

	boolean checkWritePermission() {
		SecurityManager sm = System.getSecurityManager();
		if (sm!=null) {
			return checkPermission(sm, getClassLoader(), "write");
		}
		return true; // no security manager means the check is successful
	}

	private boolean isVerbose() {
		if (verbose==null) {
			boolean verboseSet = getIsVerboseImpl();
			verbose = new Boolean(verboseSet);
			if (verboseSet) {
				printVerboseInfo("Verbose output enabled for "+getHelperType()+" id "+id);
			}
		}
		return verbose.booleanValue();
	}

	void printVerboseError(String message) {
		if (isVerbose()) {
			System.err.println("Error for "+getHelperType()+" id "+id+": "+message);
		}
	}

	void printVerboseInfo(String message) {
		if (isVerbose()) {
			System.err.println("Info for "+getHelperType()+" id "+id+": "+message);
		}
	}

	abstract String getHelperType();
}
